import {authApiBaseUrl} from "@sv/api/constants";
import logoutUser from "@sv/helpers/logoutUser";
import { updateAuthStateAction } from "@sv/lib/features/auth/authSlice";
import {AppStore, AppDispatch} from "@sv/lib/store";
import {RequestInterceptor, ResponseInterceptor, RespOverload, Options} from "@sv/types/general";

export default class ClientFetchWrapper {
  constructor(
    private getState: AppStore["getState"],
    private dispatch: AppDispatch,
    private envConfig: any,
    private reqInterceptors: RequestInterceptor[] = [],
    private respInterceptors: ResponseInterceptor[] = [],
  ) {}

  public async getWrappedFetch<T>(url: string, options: Options): Promise<RespOverload<T>> {
    console.log({url});
    let options_: Options = {
      ...options,
      headers: {
        Accept: "application/json",
        "Content-Type": "application/json",
        "D-UUID": "645545453533",
        ...options.headers,
      },
    };

    for (let reqmod of this.reqInterceptors) {
      options_ = {...options_, ...reqmod(options_)};
    }

    options_ = this._setAuthTokenHeaders(options_);

    if (options_.body && typeof options_.body !== "string") {
      options_.body = JSON.stringify(options_.body); // Stringify the body if it's not already a string
    }

    let {params = {}, ...others} = options_;
    params = new URLSearchParams(params);

    const response = await window.fetch(url + (params ? `?${params}` : ""), others);

    if (response.ok) {
      const data = await response.json();
      this._successActions(data);
      return {
        status: true,
        data,
        message: response?.statusText,
        statusCode: response.status,
      };
    }

    try {
      const errors = await response.json();
      return this._errorActions<T>(response, errors, url, options);
    } catch (error) {
      return Promise.reject({
        status: "error",
        statusCode: response.status,
        data: null,
      });
    }
  }

  private _setAuthTokenHeaders(options: Options): Options {
    const headers: Record<string, string> = {};

    try {
      const token = this.getState()?.auth?.authUser?.accessToken;
      if (token) {
        headers["Authorization"] = `bearer ${token}`;
      }
    } catch (err) {
      /* empty */
    }

    if (this.envConfig.isLocal()) console.log("HTTP request ::", options.body);

    return {...options, headers: {...options.headers, ...headers}};
  }

  private async _errorActions<T>(
    response: Response,
    errors: any,
    url: string,
    options: Options,
  ): Promise<RespOverload<T>> {
    console.log({response});
    if (response?.status === 401) {
      try {
        // Attempt to refresh the token
        const newToken = await this._refreshToken();

        console.log({newToken});

        // If token is refreshed, update the auth header and retry the original request
        if (newToken) {
          // Update options with new token
          const updatedOptions = this._setAuthTokenHeaders({
            ...options,
            headers: {
              ...options.headers,
              Authorization: `bearer ${newToken}`,
            },
          });
          return await this.getWrappedFetch<T>(url, updatedOptions);
        } else {
          // If refresh fails, dispatch logout
          logoutUser({shouldNotify: false});
        }
      } catch (refreshError) {
        // Handle refresh token failure (e.g., log out the user)
        console.log("refreshError", refreshError);
        logoutUser({shouldNotify: false});
      }
    }

    if (this.envConfig.isLocal()) console.log("HTTP error ::", errors);
    return Promise.reject({
      statusCode: response.status,
      data: errors,
      status: "error",
    });
  }

  // Helper method to refresh the token
  private async _refreshToken(): Promise<string | null> {
    try {
      const refreshToken = localStorage.getItem("refreshToken");
      const emailOrPhone = localStorage.getItem("email");

      // console.log({refreshToken, emailOrPhone});
      if (!refreshToken) throw new Error("No refresh token available");

      const response = await window.fetch(`${authApiBaseUrl}/auth/refresh-token`, {
        method: "POST",
        body: JSON.stringify({refreshToken, emailOrPhone}),
        headers: {
          "Content-Type": "application/json",
          "D-UUID": "645545453533",
        },
      });

      // console.log({response, refreshToken, emailOrPhone});

      if (response.ok) {
        const data = await response.json();
        // Assuming the response contains the new accessToken
        this.dispatch(
          updateAuthStateAction({
            accessToken: data?.accessToken ?? data?.token,
          }),
        );
        console.log({data});
        return data.accessToken;
      } else {
        throw new Error("Failed to refresh token");
      }
    } catch (err) {
      console.error("Error refreshing token:", err);
      return null;
    }
  }

  private _successActions(data: any) {
    if (this.envConfig.isLocal()) {
      console.log("HTTP response ::", data);
    }
  }
}

export type fetchWrapperType = ClientFetchWrapper["getWrappedFetch"];
